AC_DEFUN(AC_INTERNAL_ERROR,
[
	AC_MSG_ERROR([
	*** Internal error: $1.
	*** Please send this error message to temporal@gauge3d.org.])
])

AC_DEFUN(AC_GAUGE3D_TEST_PLATFORM,
[
dnl Define platform stuff ------------------------------------------------
dnl CPU test ----------

AC_MSG_CHECKING([CPU])
case ${target_cpu} in
	i*86)
		AC_MSG_RESULT([Intel x86 compatible])
		AC_DEFINE(PLATFORM_I386, 1, [Define if your system API is POSIX-compliant])
		word_size=32
		endianness=little
		;;
	*)
		AC_MSG_RESULT([Unknown: ${target_cpu}])
		AC_MSG_ERROR(
[*** Target CPU type, \"${target_cpu}\", is not supperted.
*** Only Intel ix86 processors and compatibles are supported right now.
*** If this is wrong, try using the "--target" parameter for configure.
*** (Type \"configure --help\" for more info) ])
		;;
esac

case x${word_size} in
	x32)
		AC_DEFINE(PLATFORM_32BIT, 1, [Define if you have a 32-bit CPU])
		;;
	x64)
		AC_DEFINE(PLATFORM_64BIT, 1, [Define if you have a 64-bit CPU])
		;;
	*)
		AC_INTERNAL_ERROR([bad CPU word size \"${word_size}\".])
		;;
esac

case x${endianness} in
	xlittle)
		AC_DEFINE(PLATFORM_LITTLE_ENDIAN, 1, [Define if you have a 32-bit CPU])
		;;
	xbig)
		AC_DEFINE(PLATFORM_BIG_ENDIAN, 1, [Define if you have a 64-bit CPU])
		;;
	*)
		AC_INTERNAL_ERROR([bad endianness \"${endianness}\".])
		;;
esac


dnl OS test -----------

AC_MSG_CHECKING([operating system])
case ${target_os} in
	*linux*)
		AC_MSG_RESULT([GNU/Linux])
		AC_DEFINE(PLATFORM_LINUX, 1, [Define if your operating system is Linux-based])
		is_posix=yes
		;;
	beos)
		AC_MSG_RESULT([Be OS])
		AC_DEFINE(PLATFORM_BEOS, 1, [Define if your operating system is Be OS])
		is_posix=yes
		;;
	*)
		AC_MSG_RESULT([${target_os}])
		AC_MSG_WARN([*** This operating system is not supported.  GAUGE is not likely to work.])
		
# Alright, can anyone tell me why either AC_CHECK_HEADER or AC_EGREP_CPP
# (not sure which) sets CPP and CPPFLAGS to empty strings, thus breaking
# the script?
		
dnl		AC_MSG_CHECKING([for POSIX-compliant API])
dnl		AC_CHECK_HEADER(unistd.h,
dnl		[
dnl			AC_EGREP_CPP(GAUGE3D_PLATFORM_POSIX,
dnl			[
dnl				#include <sys/types.h>
dnl				#include <unistd.h>
dnl				
dnl				#ifdef _POSIX_VERSION
dnl					GAUGE3D_PLATFORM_POSIX
dnl				#endif
dnl			],[
dnl				is_posix=yes
dnl			])
dnl		])
dnl		
dnl		if test x$is_posix = xyes; then
dnl			AC_MSG_RESULT(yes)
dnl		else
dnl			AC_MSG_RESULT(no)
			AC_MSG_ERROR([
*** Sorry, this OS is not supported.  If you would like to port GAUGE^3D to
*** this OS, please e-mail temporal@gauge3d.org.
*** If you believe this error is incorrect, try using the "--target" parameter
*** for configure. (Type \"configure --help\" for more info) ])
dnl		fi
		;;
esac

if test x$is_posix = xyes; then
	AC_DEFINE(PLATFORM_POSIX, 1, [Define if your operating system is POSIX compliant.])
fi

AC_PATH_X
if test x$have_x = xyes; then
	LIBS="$LIBS -L$x_libraries"
	AC_DEFINE(PLATFORM_X_WINDOWS, 1, [Define if you operating system uses X Windows.])
fi

dnl ----------------------------------------------------------------------
])