/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
// component header --------------------------------------------------
#include "segment.h"

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class member functions ============================================
// lifecycle ---------------------------------------------------------
GSegment::GSegment(const GVector& endPoint1, const GVector& endPoint2)
{
	mEndPoints[0] = endPoint1;
	mEndPoints[1] = endPoint2;
}

// operations --------------------------------------------------------
void
GSegment::GetLine(GLine* line) const
{
	line->u  = mEndPoints[1];
	line->u -= mEndPoints[0];
	line->v  = mEndPoints[1].X(mEndPoints[0]);
}

//Intersects returns true if the point is on the segment.  If you
// already know that it is on the same line as the segment, pass
// true for onLine to save some CPU.
inline GCoordinate
GSegment_Max(GCoordinate coord1, GCoordinate coord2)
{
	if(coord1 > coord2)
		return coord1;
	return coord2;
}

inline GCoordinate
GSegment_Min(GCoordinate coord1, GCoordinate coord2)
{
	if(coord1 > coord2)
		return coord2;
	return coord1;
}

inline bool
GSegment_Between(GCoordinate coord, GCoordinate coord1, GCoordinate coord2)
{
	if(coord > GSegment_Max(coord1, coord2))
		return false;
	if(coord < GSegment_Min(coord1, coord2))
		return false;
	return true;
}

bool
GSegment::Contains(const GVector& point)
{
	GLine line;
	GetLine(&line);
	if(!line.Contains(point))
		return false;
	
	if(GSegment_Between(point.x, mEndPoints[0].x, mEndPoints[1].x))
		if(GSegment_Between(point.y, mEndPoints[0].y, mEndPoints[1].y))
			if(GSegment_Between(point.z, mEndPoints[0].z, mEndPoints[1].z))
				return true;
	
	return false;
}

} // namespace GAUGE3D
