/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_COMMON_ERROR_H__INCLUDED__
#define __GAUGE3D_COMMON_ERROR_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_COMMON_GSTRING_H__INCLUDED__
#include "gstring.h"
#endif //redundant include guard

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GException error.h gauge3d/error.h
 *  \ingroup common
 *  \brief Thrown on fatal errors.
 *  \sa GAUGE3D_FATAL_ERROR
 *
 *  An object of type GException may be thrown when a fatal (i.e. unrecoverable)
 *  error occurs.  The main function of the program should catch this and output
 *  an error.
 */
class GAUGE3D_API GException
{
	GString mCause;
	GString mFile;
	int mLine;
	
public:
	/*! Rather than call this, use the macro \ref GAUGE3D_FATAL_ERROR to create
	 *  and throw an exception. */
	GException(GString cause, GString file, int line);
	virtual ~GException();
	
	//! A human-readable string describing the cause of the error.
	inline GString Cause() const {return mCause;}
	//! The source file from which the exception was thrown.
	inline GString File() const {return mFile;}
	//! The line of the source file from which the exception was thrown.
	inline int Line() const {return mLine;}
};

/*! \ingroup common
 *  \brief Abort execution due to a fatal error.
 *  \param CAUSE A human-readable string describing the cause of the error.
 *
 *  This macro will create and throw a GException.  It should be used only
 *  when a fatal error occurs.  The program will quit, possibly printing an
 *  error message, when this is used. */
#define GAUGE3D_FATAL_ERROR(CAUSE)         \
	throw GAUGE3D::GException(CAUSE, __FILE__, __LINE__);

} //namespace GAUGE3D

#endif //included
