/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_FILES_VIRTUALDIRECTORY_H__INCLUDED__
#define __GAUGE3D_FILES_VIRTUALDIRECTORY_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_FILES_DIRECTORY_H__INCLUDED__
#include "directory.h"
#endif //redundant include guard

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
class GVirtualDirectory;

/*! \typedef pGVirtualDirectory
 *  \ingroup files
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GVirtualDirectory GVirtualDirectory\endlink. */
typedef GSmartPointer<GVirtualDirectory> pGVirtualDirectory;

/*! \class GVirtualDirectory virtualdirectory.h gauge3d/files/virtualdirectory.h
 *  \ingroup files
 *  \brief Allows you to construct a virtual filesystem in RAM.
 *
 *  A virtual directory exists only in RAM, and files within the directory
 *  are simply pointers to any sort of object derived from GFile.  These
 *  may be disk files, temporary files, files in archives, etc. */
class GAUGE3D_API GVirtualDirectory: public GDirectory
{
	struct Private;
	Private& mPrivate;
	
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39781f65);
public:
	// lifecycle -------------------------------------------------------
	GVirtualDirectory();
	virtual ~GVirtualDirectory();
	
	// operations ------------------------------------------------------
	virtual GArray<GString> ListFiles();
	virtual GArray<GString> ListSubdirs();
	
	virtual pGFile OpenFile(GString name);
	virtual pGDirectory OpenSubdir(GString name);
	
	virtual pGFile AddFile(GString name);
	virtual pGDirectory AddSubdir(GString name);
	
	virtual void RemoveFile(GString name);
	virtual void RemoveDirectory(GString name);
	
	/*! \brief Link a file into the virtual directory.
	 *  \param name The name to give the file entry in the virtual directory.
	 *  \param file The file that the new entry should refer to.  When the
	 *              entry is requested, this same pointer will be returned. */
	void AddFile(GString name, pGFile file);
	/*! \brief Link a sub directory into the virtual directory.
	 *  \param name   The name to give the directory entry in the virtual directory.
	 *  \param subdir The directory that the new entry should refer to.  When the
	 *                entry is requested, this same pointer will be returned.  Must
	 *                be a virtual directory, to avoid confusion. */
	void AddSubdir(GString name, pGVirtualDirectory subdir);
};

} // namespace GAUGE3D

#endif //included
