/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_GRAPHICS_BILLBOARDSPRITE_H__INCLUDED__
#define __GAUGE3D_GRAPHICS_BILLBOARDSPRITE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_GRAPHICS_MODEL_H__INCLUDED__
#include "model.h"
#endif //redundant include guard


// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GBillboardSprite billboardsprite.h gauge3d/graphics/billboardsprite.h
 *  \ingroup graphics
 *  \brief A flat textured rectangle which always faces the camera. */
class GAUGE3D_API GBillboardSprite: public GModel
{
	// insulated state -------------------------------------------------
	struct Private;
	Private& mPrivate;
	
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x3a0e7e9e);
public:
	// lifecycle -------------------------------------------------------
	//! Construct a billboard sprite of the given dimentions.
	GBillboardSprite(GCoordinate width, GCoordinate height);
	//! Destructor.
	~GBillboardSprite();
	
	// access ----------------------------------------------------------
	//! Set the dimensions of the rectangle.
	void SetDimentions(GCoordinate  width, GCoordinate  height);
	//! Get the dimensions of the rectangle.
	void GetDimentions(GCoordinate* width, GCoordinate* height);
	
	// operations ------------------------------------------------------
	/*! \brief Manually set texture coordinates.
	 *  \param upperLeft  The texture coordinate corresponding to the upper
	 *                    left corner of the sprite.
	 *  \param lowerRight The texture coordinate corresponding to the lower
	 *                    right corner of the sprite. */
	void TexCoords(GVector upperLeft, GVector lowerRight);
	
	/*! \class BillboardAnimState billboardsprite.h gauge3d/graphics/billboardsprite.h
	 *  \brief The animation state for a GBillboardSprite.
	 *
	 *  At this time, this class does absolutely nothing, as billboard sprites
	 *  are not animated. */
	class GAUGE3D_API BillboardAnimState: public GModel::AnimState
	{
		GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x3a0e7eac);
	public:
		virtual ~BillboardAnimState();

#ifdef DOXYGEN
		//! This interface is not done yet.
		Under Construction();
#endif
		// control info ------------------------------------------------
		//TODO: Set animation frame.
		// Set texture coords?
	};
	/*! \typedef pBillboardAnimState
	 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
	 *         \link BillboardAnimState BillboardAnimState\endlink. */
	typedef GSmartPointer<BillboardAnimState> pBillboardAnimState;
	
	virtual pAnimState CreateAnimState();
	
	// rendering info ----------------------------------------------
	pRenderInfo CreateRenderInfo();
	
	class BillboardRenderInfo;
	friend class BillboardRenderInfo;
};
/*! \typedef pGBillboardSprite
 *  \ingroup graphics
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GBillboardSprite GBillboardSprite\endlink. */
typedef GSmartPointer<GBillboardSprite> pGBillboardSprite;

} //namespace GAUGE3D

#endif //included
