/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
// component header --------------------------------------------------
#include "timer.h"

// system ------------------------------------------------------------
#ifdef HAVE_GETTIMEOFDAY
#include <sys/time.h>
#endif // HAVE_GETTIMEOFDAY

#ifdef PLATFORM_POSIX
#include <unistd.h>
#include <time.h>
#endif // PLATFORM_POSIX

#ifdef PLATFORM_WINDOWS
#include <windows.h>
#endif

// external packages -------------------------------------------------
// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

namespace {

void GetSysTime(uint* sec, int* msec)
{
#ifdef HAVE_GETTIMEOFDAY
	timeval t;

	gettimeofday(&t, 0);
	*sec  = t.tv_sec;
	*msec = t.tv_usec / 1000;
#elif defined(PLATFORM_WINDOWS)
	int t = GetTickCount();
	*sec  = t / 1000;
	*msec = t % 1000;
#else
#error "Don't know how to read the system timer."
#endif // HAVE_GETTIMEOFDAY
}

static uint gStartSec = 0;
static int gStartMSec = 0;

} //anonymous namespace

// class member functions ============================================
void
GTimer::Init(tTime startTime)
{
	GetSysTime(&gStartSec, &gStartMSec);
}

tTime
GTimer::Time()
{
	uint curSec;
	int curMSec;

	GetSysTime(&curSec, &curMSec);
	curSec  -= gStartSec;
	curMSec -= gStartMSec;
	
	//Of course, curMSec may be negative.  :)
	return curSec + curMSec / 1000.0;
}

void
GTimer::Sleep(tTime length)
{
#ifdef PLATFORM_POSIX
	timespec timeSpec;

	timeSpec.tv_sec = (time_t)(length);
	timeSpec.tv_nsec = (long)((length - timeSpec.tv_sec) * 1000000000);
	
	nanosleep(&timeSpec, NULL);
#elif defined(PLATFORM_WINDOWS)
	::Sleep((uint)(length * 1000));
#else
#error "Don't know how to sleep without POSIX."
#endif // PLATFORM_POSIX
}

} // namespace GAUGE3D
