/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_PLUGINS_FILELOADER_H__INCLUDED__
#define __GAUGE3D_PLUGINS_FILELOADER_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
#ifndef __GAUGE3D_FILES_FILE_H__INCLUDED__
#include "gauge3d/files/file.h"
#endif //redundant include guard

// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GFileLoader fileloader.h gauge3d/plugins/fileloader.h
 *  \ingroup plugins
 *  \brief Base class for plugins that load and save particular file formats.
 *
 *  This class is used when a program needs better control over loading
 *  and saving particular formats.  If you simply want to load a file,
 *  you probably don't need this class.  Just use GObjectLoader instead.
 *
 *  This class will probably be removed in a future version of GAUGE.
 *  It will be replaced by a better system.
 */
class GAUGE3D_API GFileLoader: public GObject
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39ecbfd8);
public:
	// lifecycle -------------------------------------------------------
	virtual ~GFileLoader();
	
	// operations ------------------------------------------------------
	virtual GString FileType() = 0;
	
	virtual pGObject Load(pGFile file) = 0;
	virtual bool     Save(pGFile file, pGObject object) = 0;
	virtual bool     CanSave(pGObject object) = 0;
	
	
	struct ChunkHeader
	{
		uint32 mType;
		uint32 mSize;
	};
	
	virtual pGObject Read(pGIStream input) = 0;
	virtual int      Write(pGOStream output, pGObject object) = 0;
};
/*! \typedef pGFileLoader
 *  \ingroup plugins
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GFileLoader GFileLoader\endlink. */
typedef GSmartPointer<GFileLoader> pGFileLoader;

} //namespace GAUGE3D

#endif //included
