/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_FILES_CONFIGFILE_H__INCLUDED__
#define __GAUGE3D_FILES_CONFIGFILE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include <gauge3d/stddefs.h>
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
#ifndef __GAUGE3D_COMMON_GSTRING_H__INCLUDED__
#include <gauge3d/common/gstring.h>
#endif //redundant include guard

// local package -----------------------------------------------------
#ifndef __GAUGE3D_FILES_VIRTUALDIRECTORY_H__INCLUDED__
#include "virtualdirectory.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GConfigFile configfile.h gauge3d/files/configfile.h
 *  \ingroup files
 *  \brief Access to a section-key-data config file.
 *
 *  GConfigFile is <b>not</b> derived from GFile.  Heck, GConfigFile is
 *  not even an instantiable class (it is fully static).  You give it a GFile
 *  to use.  It treats that file as a config file in a similar format
 *  to that of win.ini on Windows systems, but with some enhancements.
 *  Each sections starts
 *  with a section title in square brackets, and ends with the same
 *  section title with a '/' infront of it, again in square brackets.
 *  Each line in the section has the format key=data.  You look up
 *  settings by the key.  Also note that sections can be embedded within
 *  other sections, and also that the file itself is a section, so if
 *  you don't need separate sections, you don't have to use them.
 *
 *  GConfigFile is actually a static class.  You should not create an
 *  instance of it.  Instead, use the functions within GConfigFile to
 *  load the configuration into a virtual directory.  Then, you can
 *  navigate sections of the config file just as you would directories
 *  in a filesystem.  Each key is a file, and the corresponding data
 *  can be retrieved from that file by casting the file to a string.
 *  (You can use normal file i/o methods to edit the configuration as
 *  well, but casting to a string will be much faster.) */
class GAUGE3D_API GConfigFile
{
public:
	// operations ------------------------------------------------------
	/*! \brief Loads the file into a new virtual directory and returns it.
	 *  \returns A pointer to the new virtual directory, or NULL if the file
	 *           could not be read. */
	static pGVirtualDirectory Load(pGFile file);
	
	/*! \brief Saves the configuration to the given file.
	 *  \returns True if successful, or false if the file was not writable. */
	static bool Save(pGFile file, pGVirtualDirectory config);
};

} // namespace GAUGE3D

#endif //included
