/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_FILES_STRINGFILE_H__INCLUDED__
#define __GAUGE3D_FILES_STRINGFILE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include <gauge3d/stddefs.h>
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_FILES_FILE_H__INCLUDED__
#include "file.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GStringFile stringfile.h gauge3d/files/stringfile.h
 *  \ingroup files
 *  \brief File represented by a string.
 *
 *  The data in a GStringFile is stored as a GString.  The idea here
 *  is that we want a file which can easily be cast to a string.  This
 *  is used heavily by GConfigFile, which loads config files into a
 *  virtual filesystem where directories are sections of the config
 *  file and files are individual settings. */
class GAUGE3D_API GStringFile: public GFile
{
	GString mData;
	GString mName;
	
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39ce4d79);
public:
	// lifecycle -------------------------------------------------------
	//! Construct an empty file.
	GStringFile();
	//! Construct a GStringFile with the given contents.
	GStringFile(GString data);
	//! Destructor.
	virtual ~GStringFile();
	
	// operations ------------------------------------------------------
	virtual int Size() {return mData.Length();}
	virtual bool Truncate() {mData = GString(); return true;}
	virtual pGRawIStream GetRawInputStream();
	virtual pGRawOStream GetRawOutputStream();
	virtual pFileMap Map(int offset, int size, int flags);
	virtual GString MapToDisk();
	virtual GString Name() {return mName;}
	//! Set the internal name of the file.
	inline void Name(GString name) {mName = name;}
	//! Set the data contained by the file.
	inline void Data(GString data) {mData = data;}
	
	virtual operator GString() { return mData; }
};
/*! \typedef pGStringFile
 *  \ingroup files
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GStringFile GStringFile\endlink. */
typedef GSmartPointer<GStringFile> pGStringFile;

} //namespace GAUGE3D

#endif //included
