/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_RENDERER_RENDERER_H__INCLUDED__
#define __GAUGE3D_RENDERER_RENDERER_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include <gauge3d/stddefs.h>
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_RENDERER_DISPLAY_H__INCLUDED__
#include "display.h"
#endif
#ifndef __GAUGE3D_RENDERER_SPRITE_H__INCLUDED__
#include "sprite.h"
#endif
#ifndef __GAUGE3D_RENDERER_LIGHT_H__INCLUDED__
#include "light.h"
#endif
#ifndef __GAUGE3D_RENDERER_TEXTURE_H__INCLUDED__
#include "texture.h"
#endif
#ifndef __GAUGE3D_RENDERER_SCENE_H__INCLUDED__
#include "scene.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GRenderer renderer.h gauge3d/renderer/renderer.h
 *  \ingroup renderer
 *  \brief Represents a hardware 3D rendering device.
 *
 *  This class is the one that is exported by rendering plugins.  Given
 *  a GRenderer, you can do anything graphics-wise.  GRenderer does not
 *  do anything itself, but it acts as a vendor for other rendering
 *  classes which you can use to get the real work done. */
class GAUGE3D_API GRenderer: public GObject
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39781feb);
public:
	// lifecycle -------------------------------------------------------
	virtual ~GRenderer();
	
	// operations ------------------------------------------------------
	/*! \brief   Create a new GDisplay associated with this renderer.
	 *  \remarks Not all rendering API's (or hardware) support creating more than one
	 *           display.  Be prepared for NewDisplay() to return NULL if
	 *           you have already created a display with this renderer. */
	virtual pGDisplay NewDisplay() = 0;
	//! Create a new GTexture associated with this renderer.
	virtual pGTexture NewTexture() = 0;
	//! Create a new GScene associated with this renderer.
	virtual pGScene   NewScene()   = 0;
};
/*! \typedef pGRenderer
 *  \ingroup renderer
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GRenderer GRenderer\endlink. */
typedef GSmartPointer<GRenderer> pGRenderer;

} //namespace GAUGE3D

#endif //included
