/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_RENDERER_SCENE_H__INCLUDED__
#define __GAUGE3D_RENDERER_SCENE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include <gauge3d/stddefs.h>
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_RENDERER_SPRITE_H__INCLUDED__
#include "sprite.h"
#endif

#ifndef __GAUGE3D_RENDERER_LIGHT_H__INCLUDED__
#include "light.h"
#endif

#ifndef __GAUGE3D_RENDERER_CAMERA_H__INCLUDED__
#include "camera.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GScene scene.h gauge3d/renderer/scene.h
 *  \ingroup renderer
 *  \brief Represents a 3D scene.
 *
 *  A scene is basically a collection of sprites and lights within a world
 *  map.  You can create multiple cameras looking in on a scene. */
class GAUGE3D_API GScene: public GObject
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39781fc2);
public:
	// lifecycle -------------------------------------------------------
	virtual ~GScene();
	
	// operations ------------------------------------------------------
	//! Create a new GSprite inside the scene.
	virtual pGSprite NewSprite() = 0;
	//! Create a new GLight inside the scene.
	virtual pGLight  NewLight()  = 0;
	//! Create a new GCamera looking into the scene.
	virtual pGCamera NewCamera() = 0;
};
/*! \typedef pGScene
 *  \ingroup renderer
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GScene GScene\endlink. */
typedef GSmartPointer<GScene> pGScene;

} //namespace GAUGE3D

#endif //included
