/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_RENDERER_TEXTURE_H__INCLUDED__
#define __GAUGE3D_RENDERER_TEXTURE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include <gauge3d/stddefs.h>
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
#ifndef __GAUGE3D_GRAPHICS_SURFACE_H__INCLUDED__
#include <gauge3d/graphics/surface.h>
#endif

// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GTexture texture.h gauge3d/renderer/texture.h
 *  \ingroup renderer
 *  \brief Represents a texture loaded into the rendering device.
 *
 *  A GTexture can be more than just a color map.  It can be a complete
 *  surface descriptor, including gloss map, elevation map, bump map,
 *  etc. */
class GAUGE3D_API GTexture: public GObject
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39781fe3);
public:
	// lifecycle -------------------------------------------------------
	virtual ~GTexture();
	
	// operations ------------------------------------------------------
	/*! \brief Sets the surface data for the texture.
	 *
	 * \note Most hardware requires textures to have power-of-two dimentions.
	 *       If you specify a surface which contains images that break this
	 *       rule, the renderer will scale the image up to the next power-of-two
	 *       dimentions for you.  So, don't worry abuot it unless you have load
	 *       time problems. */
	virtual void Surface(pGSurface surface) = 0;
	
	/*! \brief Sets the color and (possibly) opacity data for the texture.
	 *
	 * \remarks This is basically the same as specifying a surface which
	 *          only contains this one image interpreted as a color-opacity
	 *          map (or just color, if the image has no alpha channel). */
	virtual void Image(pGImage image) = 0;
};
/*! \typedef pGTexture
 *  \ingroup renderer
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GTexture GTexture\endlink. */
typedef GSmartPointer<GTexture> pGTexture;

} // namespace GAUGE3D

#endif //included
