/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Steven Haworth
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: gifimage.h
 *      Package: GIF image file plugin

 *    Component: ImgPak
 *  Description: Loads GIF Images
 * Dependencies: GImage
 *               GFile
 */

#ifndef __GAUGE3D_PLUGINS_IMAGE_GIFIMAGE_H__INCLUDED__
#define __GAUGE3D_PLUGINS_IMAGE_GIFIMAGE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
#include <string.h>

// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_GRAPHICS_IMAGE_H__INCLUDED__
#include "gauge3d/graphics/image.h"
#endif

#ifndef __GAUGE3D_FILES_FILE_H__INCLUDED__
#include "gauge3d/files/file.h"
#endif

// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
class GpGifImage: public GImage
{
	GAUGE3D_SET_UID(0x00000048, 0x2168a11d, 0x39825a96)
public:
	// lifecycle -------------------------------------------------------
	GpGifImage(pGFile giffile);
	virtual ~GpGifImage();
	
	//Format() fills in the argument with the format info.
	virtual FormatDesc Format() { return mFormat; }
	
	//Data() retuns the raw image data in the format given by GetFormat().
	//mipmapLevel is the mip map to retun.  For example, for a 64x64 image,
	// level 0 is 64x64, 1 is 32x32, 2 is 16x16, etc.  Images that don't provide
	// mip maps should ignore this.
	virtual GArray<uint8> Data(int mipmapLevel = 0);
	
	virtual GArray<uint8> EncodedData(int mipmapLevel = 0) {return GArray<uint8>();}
	
	// access ----------------------------------------------------------
	inline bool ReadOk() {return mReadOk;}
	
private:
	bool mReadOk;   //whether or not the file was successufully read.

	int mVersion;
	uint16 mWidth,mHeight;
	bool mInterlaced;
	FormatDesc mFormat;
	static unsigned short mCodeMask[13];
	GArray<uint8> mData;
	int mGroup,mLine;

	struct {
		int    mSize;
		uint8 *mpData;
		bool   mHasTransparentColor;
		uint8  mTransparentColor;
	} mColorTable;

	int GetByte(GIStream& input) { unsigned char data; input.Read(&data,1); return static_cast<int>(data); }
	bool Load(pGFile gifFile);
	bool Decoder(GIStream& input,int lineWidth);
	int GetNextCode(GIStream& input,int curSize,int& bytesLeft,int& bitsLeft,unsigned char& curByte);
	void OutLine(unsigned char *pixels,int length);
};

} //namespace GAUGE3D

#endif //included
