/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Steven Haworth
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: imgplugin.cpp
 *      Package: Plugin
 *    Component: ImgPak
 *  Description:
 * Dependencies: GPlugin
 */

// includes ==========================================================
// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_PLUGIN_H__INCLUDED__
#include "gauge3d/plugins/plugin.h"
#endif

// system ------------------------------------------------------------
// extenal packages -------------------------------------------------
// local package -----------------------------------------------------
#include "bmpimage.h"
#include "gifimage.h"
#include "pcximage.h"
#include "tgaimage.h"

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
class GpImgPakPlugin: public GPlugin
{
	GAUGE3D_SET_UID(0x00000048, 0x2168a11d, 0x3982563a)
public:
	// lifecycle -------------------------------------------------------
	GpImgPakPlugin() {};
	virtual ~GpImgPakPlugin() {};
	
	// inquiry ---------------------------------------------------------
	//these retun the human-readable name and description of the plugin.
	virtual GString Name() {return "Image Pak";}
	virtual GString Description()
		{return "Loads several image file formats.";}
	
	//InUse retuns true if the plugin is being used.  Otherwise, false.
	// The object loader uses this to determine when it is safe to unload
	// a plugin from RAM.
	virtual bool InUse() {return true;}  //FIXME
	
	// operations ------------------------------------------------------
	virtual pGObject CreateObject(const GString& className)
	{
		return NULL;   //we don't serve any non-datafile plugins.
	}
	
	//When loading a file, if the plugin finds that it can't load the thing,
	// it should retun NULL.
	virtual pGObject CreateObject(const GString& className, pGFile file)
	{
		if(className == "GpGifImage") {
			GpGifImage *ret = new GpGifImage(file);
			if(!ret->ReadOk()) {
				delete ret;
				return NULL;
			}
			else {
				return ret;
			}
		}
		else if(className == "GpPcxImage") {
			GpPcxImage *ret = new GpPcxImage(file);
			if(!ret->ReadOk()) {
				delete ret;
				return NULL;
			}
			else {
				return ret;
			}
		}
		else if(className == "GpTgaImage") {
			GpTgaImage *ret = new GpTgaImage(file);
			if(!ret->ReadOk()) {
				delete ret;
				return NULL;
			}
			else {
				return ret;
			}
		}
		else if(className == "GpBmpImage") {
			GpBmpImage *ret = new GpBmpImage(file);
			if(!ret->ReadOk()) {
				delete ret;
				return NULL;
			}
			else {
				return ret;
			}
		}
		return NULL;   //we don't serve those.
	}
	
	//The plugin tells the object loader what objects are available from
	// it by giving it an array of ObjectInfo objects.  Each one gives
	// info for one of the available objects.
	//ObjectList() retuns ObjectInfos for every object available from
	// this plugin.
	virtual GArray<ObjectInfo> ObjectList()
	{
		GArray<ObjectInfo> info(4);
		
		info[0].mClassname      = "GpGifImage";
		info[0].mName           = "GIF Image";
		info[0].mDescription    = "Loads GIF images.";
		info[0].mBaseClasses    = GArray<GString>(1);
		info[0].mBaseClasses[0] = "GImage";
		info[0].mFileTypes      = GArray<GString>(1);
		info[0].mFileTypes[0]   = "gif";

		info[1].mClassname      = "GpPcxImage";
		info[1].mName           = "PCX Image";
		info[1].mDescription    = "Loads PCX images.";
		info[1].mBaseClasses    = GArray<GString>(1);
		info[1].mBaseClasses[0] = "GImage";
		info[1].mFileTypes      = GArray<GString>(1);
		info[1].mFileTypes[0]   = "pcx";

		info[2].mClassname      = "GpBmpImage";
		info[2].mName           = "BMP Image";
		info[2].mDescription    = "Loads BMP images.";
		info[2].mBaseClasses    = GArray<GString>(1);
		info[2].mBaseClasses[0] = "GImage";
		info[2].mFileTypes      = GArray<GString>(1);
		info[2].mFileTypes[0]   = "bmp";

		info[3].mClassname      = "GpTgaImage";
		info[3].mName           = "TGA Image";
		info[3].mDescription    = "Loads TGA images.";
		info[3].mBaseClasses    = GArray<GString>(1);
		info[3].mBaseClasses[0] = "GImage";
		info[3].mFileTypes      = GArray<GString>(1);
		info[3].mFileTypes[0]   = "tga";
		
		return info;
	}
};

GAUGE3D_DECLARE_INTERFACES(GpImgPakPlugin)
	GAUGE3D_DECLARE_BASECLASS(GPlugin)
GAUGE3D_END_INTERFACES

GAUGE3D_EXPORT_PLUGIN(GpImgPakPlugin)

} //namespace GAUGE3D
