/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2001 Will Newton
 *               based on code by Steven Haworth
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: pngimage.h
 *      Package: PNG image file plugin

 *    Component: PngImg
 *  Description: Loads PNG Images
 * Dependencies: GImage
 *               GFile
 */

#ifndef __GAUGE3D_PLUGINS_IMAGE_PNGIMAGE_H__INCLUDED__
#define __GAUGE3D_PLUGINS_IMAGE_PNGIMAGE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
#include <string.h>

#include <png.h>

// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_GRAPHICS_IMAGE_H__INCLUDED__
#include "gauge3d/graphics/image.h"
#endif

#ifndef __GAUGE3D_FILES_FILE_H__INCLUDED__
#include "gauge3d/files/file.h"
#endif

// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

#define INPUT_BUF_SIZE  4096 

// class definition ==================================================
class GpPngImage: public GImage
{
	GAUGE3D_SET_UID(0x00000040, 0x05a64f2f, 0x3a78f372);
	public:
	// lifecycle -------------------------------------------------------
	GpPngImage(pGFile pngfile);
	virtual ~GpPngImage();
	
	virtual FormatDesc Format() { return mFormat; }
	
	//Data() retuns the raw image data in the format given by Format().
	//mipmapLevel is the mip map to retun.  For example, for a 64x64 image,
	// level 0 is 64x64, 1 is 32x32, 2 is 16x16, etc.  Images that don't provide
	// mip maps should ignore this.
	virtual GArray<uint8> Data(int mipmapLevel = 0);
	
	virtual GArray<uint8> EncodedData(int mipmapLevel = 0) {return GArray<uint8>();}
	
	// access ----------------------------------------------------------
	inline bool ReadOk() { return mReadOk; }
	
private:
	bool mReadOk;   //whether or not the file was successufully read.

	FormatDesc mFormat;
	GArray<uint8> mData;

};

} //namespace GAUGE3D

#endif //included
