/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: 3dstudio.h
 *      Package:
 *    Component:
 *  Description:
 * Dependencies:
 */

#ifndef __GAUGE3D_PLUGINS_MODEL_3DSTUDIO_3DSTUDIO_H__INCLUDED__
#define __GAUGE3D_PLUGINS_MODEL_3DSTUDIO_3DSTUDIO_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
#include <vector>
using namespace std;

// GAUGE^3D ----------------------------------------------------------
#include "gauge3d/graphics/supermodel.h"
#include "gauge3d/files/file.h"

// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
class Gp3DSModel: public GSuperModel
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39ed35f9);
public:
	Gp3DSModel(pGFile file);
	virtual ~Gp3DSModel();
	
	inline bool IsOpen() {return mOpen;}
	
private:
	bool mOpen;
	
	void ReadTriMesh(GIStream& input, int size);
	
	Model     mModel;
	Mesh      mMesh;
	MeshFrame mFrame;
	
	enum
	{
		MAIN3DS     = 0x4D4D,
		EDIT3DS     = 0x3D3D,
		EDIT_OBJECT = 0x4000,
		OBJ_TRIMESH = 0x4100,
		
		TRI_VERTEXL = 0x4110,
		TRI_MAPPINGCOORDS = 0x4140,
		TRI_FACEL1  = 0x4120,
		TRI_SMOOTH  = 0x4150,
		TRI_LOCAL   = 0x4160
	};
	
	vector<GVector> mVertices;
	vector<GVector> mNormals;
	vector<GVector> mTexCoords;
	vector<uint16>  mIndices;
};

} //namespace GAUGE3D

#endif //included
