/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: gauge3d-mdl.cpp
 *      Package:
 *    Component:
 *  Description:
 * Dependencies:
 */

// includes ==========================================================
// component header --------------------------------------------------
#include "gauge3d-mdl.h"

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {
namespace GMDL {

// class member functions ============================================
GpGMDLModel::GpGMDLModel(Model* model)
{
	ModelData(model);
}

GpGMDLModel::~GpGMDLModel()
{
	//You think this code is disgusting now?  You should have seen it before!
	Model* model = ModelData();
	int i, k;
	
	//delete meshes
	for(k = 0; k < model->mNumMeshes; k++)
	{
		//delete frames
		for(i = 0; i < model->mMeshes[k].mNumFrames; i++)
		{
			//delete frame contents
			
			//the frame will have interleaved vertices and normals
			if(model->mMeshes[k].mFrames[i].mVertices)
				delete [] model->mMeshes[k].mFrames[i].mVertices;
			if(model->mMeshes[k].mFrames[i].mTexCoords)
				delete [] model->mMeshes[k].mFrames[i].mTexCoords;
		}
		if(model->mMeshes[k].mNumFrames > 0)
			delete [] model->mMeshes[k].mFrames;
		
		//delete edge reductions
		if(model->mMeshes[k].mEdgeReductions)
		{
			//mChangedIndices is one big array which all of the edge reductions use
			delete [] model->mMeshes[k].mEdgeReductions->mChangedIndices;
			delete [] model->mMeshes[k].mEdgeReductions;
		}
		
		//delete index list
		if(model->mMeshes[k].mNumIndices > 0)
			delete [] model->mMeshes[k].mIndices;
	}
	if(model->mNumMeshes > 0)
		delete [] model->mMeshes;
	
	delete model;
}


GAUGE3D_DECLARE_INTERFACES(GpGMDLModel)
	GAUGE3D_DECLARE_BASECLASS(GSuperModel)
GAUGE3D_END_INTERFACES

} //namespace GMDL
} //namespace GAUGE3D
