/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: loader.h
 *      Package:
 *    Component:
 *  Description:
 * Dependencies:
 */

#ifndef __GAUGE3D_PLUGINS_MODEL_GAUGE3D_MDL_LOADER_H__INCLUDED__
#define __GAUGE3D_PLUGINS_MODEL_GAUGE3D_MDL_LOADER_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// GAUGE^3D ----------------------------------------------------------
#include "gauge3d/plugins/fileloader.h"

// local package -----------------------------------------------------
#include "gauge3d-mdl.h"

// begin namespace ===================================================
namespace GAUGE3D {
namespace GMDL {

// class definition ==================================================
class GpGMDLLoader: public GFileLoader
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39ecc9c3);
public:
	// lifecycle -------------------------------------------------------
	virtual ~GpGMDLLoader();
	
	// operations ------------------------------------------------------
	virtual GString  FileType();
	virtual bool     CanSave(pGObject object);
	
	virtual pGObject Load(pGFile file);
	virtual bool     Save(pGFile file, pGObject object);
	
	
	virtual pGObject Read(pGIStream input);
	virtual int      Write(pGOStream output, pGObject object);
	
	
	
	
	inline static void SkipChunk(GIStream* is, ChunkHeader chunk)
	{
		is->Seek(chunk.mSize - sizeof(ChunkHeader), GIStream::CURRENT);
	}
	
	enum
	{
		CHUNK_MODEL     = 0x474D444C,   //ASCII code for "GMDL" -- works as a magic number
		CHUNK_MESH      = 0x00010001,
		CHUNK_MESHFRAME = 0x00010002,
		
		CHUNK_VERTEX_POSITIONS = 0x00010003,
		CHUNK_VERTEX_NORMALS   = 0x00010004,
		CHUNK_VERTEX_TEXCOORDS = 0x00010005,
		
		CHUNK_EDGE_REDUCTIONS = 0x00010006
	};
	
	typedef GSuperModel::Model Model;
	typedef GSuperModel::Mesh Mesh;
	typedef GSuperModel::MeshFrame MeshFrame;
	typedef GSuperModel::EdgeReduction EdgeReduction;
	
	static void ReadModel(Model* model, GIStream* input, int size);
	static int  WriteModel(Model* model, GOStream* output);
	
	static void ReadMesh(Mesh* mesh, GIStream* input, int size);
	static int  WriteMesh(Mesh* mesh, GOStream* output);
	
	static void ReadMeshFrame(MeshFrame* frame, GIStream* input, int size, int numVertices);
	static int  WriteMeshFrame(MeshFrame* frame, GOStream* output, int numVertices);
	
//	static void Read(* , GIStream* input, ChunkHeader chunk);
//	static int  Write(* , GOStream* output);
};

inline GIStream& operator>>(GIStream& is, GpGMDLLoader::ChunkHeader& chunk)
{
	return is >> chunk.mType >> chunk.mSize;
}
inline GOStream& operator<<(GOStream& os, GpGMDLLoader::ChunkHeader& chunk)
{
	return os << chunk.mType << chunk.mSize;
}

} //namespace GMDL
} //namespace GAUGE3D

#endif //included
