/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: plugin.cpp
 *      Package:
 *    Component:
 *  Description:
 * Dependencies:
 */

// includes ==========================================================
// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_PLUGIN_H__INCLUDED__
#include "gauge3d/plugins/plugin.h"
#endif

// local package -----------------------------------------------------
#include "loader.h"

// begin namespace ===================================================
namespace GAUGE3D {
namespace GMDL {

// class definition ==================================================
class GpGMDLPlugin: public GPlugin
{
	GpGMDLLoader mLoader;
	
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x397ec635)
public:
	// lifecycle -------------------------------------------------------
	GpGMDLPlugin() {};
	virtual ~GpGMDLPlugin() {};
	
	// inquiry ---------------------------------------------------------
	//these return the human-readable name and description of the plugin.
	virtual GString Name() {return "GAUGE^3D GMDL Model";}
	virtual GString Description()
		{return "Loads and saves models in the GAUGE^3D GMDL model format.";}
	
	//InUse returns true if the plugin is being used.  Otherwise, false.
	// The object loader uses this to determine when it is safe to unload
	// a plugin from RAM.
	virtual bool InUse() {return true;}  //FIXME
	
	// operations ------------------------------------------------------
	virtual pGObject CreateObject(const GString& className)
	{
		if(className == "GpGMDLLoader")
			return new GpGMDLLoader;
		else
			return NULL;   //we don't serve those.
	}
	
	//When loading a file, if the plugin finds that it can't load the thing,
	// it should return NULL.
	virtual pGObject CreateObject(const GString& className, pGFile file)
	{
		if(className == "GpGMDLModel")
			return mLoader.Load(file);
		else
			return NULL;   //we don't serve those.
	}
	
	//The plugin tells the object loader what objects are available from
	// it by giving it an array of ObjectInfo objects.  Each one gives
	// info for one of the available objects.
	//ObjectList() returns ObjectInfos for every object available from
	// this plugin.
	virtual GArray<ObjectInfo> ObjectList()
	{
		GArray<ObjectInfo> info(2);
		
		info[0].mClassname   = "GpGMDLModel";
		info[0].mName        = "GAUGE^3D GMDL Model";
		info[0].mDescription = "Loads models in the GAUGE^3D GMDL model format.";
		
		info[0].mBaseClasses    = GArray<GString>(2);
		info[0].mBaseClasses[0] = "GSuperModel";
		info[0].mBaseClasses[1] = "GModel";
		info[0].mFileTypes      = GArray<GString>(1);
		info[0].mFileTypes[0]   = "gmdl";
		
		info[1].mClassname   = "GpGMDLLoader";
		info[1].mName        = "GAUGE^3D GMDL Loader/Saver";
		info[1].mDescription = "Loads and saves models in the GAUGE^3D GMDL model format.";
		
		info[1].mBaseClasses    = GArray<GString>(1);
		info[1].mBaseClasses[0] = "GFileLoader";
		info[1].mKeywords       = GArray<GString>(3);
		info[1].mKeywords[0]    = ".gmdl";
		info[1].mKeywords[1]    = "GSuperModel";
		info[1].mKeywords[2]    = "GpGMDLModel";
		
		return info;
	}
};

GAUGE3D_DECLARE_INTERFACES(GpGMDLPlugin)
	GAUGE3D_DECLARE_BASECLASS(GPlugin)
GAUGE3D_END_INTERFACES

GAUGE3D_EXPORT_PLUGIN(GpGMDLPlugin)

} //namespace GMDL
} //namespace GAUGE3D
