/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Direct3D 7 Rendering Plugin
 * Copyright (C) 2000 Steven Haworth
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: d3d7light.h
 *      Package: Direct3D 7 Rendering Plugin
 *    Component: Light
 *  Description: D3D implementation of a light
 * Dependencies: 
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_DIRECT3D7_D3D7LIGHT_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_DIRECT3D7_D3D7LIGHT_H__INCLUDED__

#ifndef DIRECTINPUT_VERSION
#define DIRECTINPUT_VERSION 0x0700
#define DIRECT3D_VERSION    0x0700
#endif

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
#ifndef __SYSTEM_DDRAW_H__INCLUDED__
#define D3D_OVERLOADS
#include <ddraw.h>
#define __SYSTEM_DDRAW_H__INCLUDED__
#endif //redundant include guard

#ifndef __SYSTEM_D3D_H__INCLUDED__
#include <d3d.h>
#define __SYSTEM_D3D_H__INCLUDED__
#endif //redundant include guard

// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_LIGHT_H__INCLUDED__
#include "gauge3d/renderer/light.h"
#endif //redundant include guard

// local package -----------------------------------------------------
#ifndef __LOCAL_DXERROR_H__INCLUDED__
#include "dxerror.h"
#define __LOCAL_DXERROR_H__INCLUDED__
#endif

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpD3D7Light;
typedef GSmartPointer<GpD3D7Light> pGpD3D7Light;
class GpD3D7Light: public GLight
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39782270);
public:
	GpD3D7Light(pGpD3D7Light* link,LPDIRECT3DDEVICE7 device);
	virtual ~GpD3D7Light();
	
	void Register(int num);
	
	// operations ------------------------------------------------------
	//All lights start out hidden.  After you have cached the light, call Show() to
	// enable it.  Show and Hide are a good way to turn lights on and off.
	virtual void Show();
	virtual void Hide();
	//Call Remove to permenantly remove the light from the scene.
	virtual void Remove();
	
	// access ----------------------------------------------------------
	//The origin is the location of the light.
	virtual void Position(const GVector& position);
	
	//Instead of an origin, you can specify a direction for an infinite light source.
	virtual void Direction(const GVector& direction);
	
	//Use these to set the color of the light.
	virtual void Color(float red, float green, float blue);
	virtual void Color(const float color[]);   //3-element array: RGB
	
	//LightProps sets the light properties.
	virtual void LightProps(float ambient, float diffuse, float specular = 0.0);
	virtual void LightProps2v(const float lightProps[]);  //ambient and diffuse
	virtual void LightProps3v(const float lightProps[]);  //ambient, diffuse, and specular
	
	//Attenuation allows you to set linear and quadratic attenuation factors.
	//(this makes the light get dimmer as you get further away)
	virtual void Attenuation(float linear, float quadratic = 0.0);
	
	//Spotlight makes the light into a spotlight pointing in the specified
	// direction and having the specified width (angle in degrees).  exponent
	// specifies how focused the spotlight is.  To disable, specify -1.0 for width.
	virtual void Spotlight(const GVector& direction, float exponent, float width);
	
	
	inline pGpD3D7Light Next() {return mNext;}
	
private:
	GError *mError;
	bool mEnabled;
	
	GVector mPosition;
	bool mDirectional;
	float mColor[3];
	float mLightProps[3];
	float mAttenuation[2];
	
	GVector mSpotDirection;
	float mSpotExponent;
	float mSpotWidth;       //angle in degrees

	LPDIRECT3DDEVICE7 mDevice;

	pGpD3D7Light  mNext;
	pGpD3D7Light* mPrev;      //actually a pointer to the previous' mNext field
};

} // namespace Plugins
} // namespace GAUGE3D

#endif //included
