/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Direct3D 7 Rendering Plugin
 * Copyright (C) 2000 Steven Haworth
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: d3d7scene.cpp
 *      Package: Direct3D 7 Rendering Plugin
 *    Component: Scene
 *  Description: Basically just a list of sprites and lights
 * Dependencies: sprites, lights, etc.
 */

// includes ==========================================================
// component header --------------------------------------------------
#include "d3d7scene.h"

// system ------------------------------------------------------------
// GAUGE^3D ----------------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_RENDERER_DIRECT3D_D3D7CAMERA_H__INCLUDED__
#include "d3d7camera.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class member functions ============================================
// lifecycle ---------------------------------------------------------
GpD3D7Scene::GpD3D7Scene(IDirect3DDevice7 *device) :
	mDevice(device)
{
}
GpD3D7Scene::~GpD3D7Scene()
{
}

// operations --------------------------------------------------------
//Add lights and sprites to the scene.
pGSprite
GpD3D7Scene::NewSprite()
{
	return new GpD3D7Sprite(&mSpriteList,mDevice);
}

pGLight
GpD3D7Scene::NewLight()
{
	return new GpD3D7Light(&mLightList,mDevice);
}

pGCamera
GpD3D7Scene::NewCamera()
{
	return new GpD3D7Camera(this,mDevice);
}

//Render the scene, assuming the projection and modelview matrices have
// already been set up so that the scene can be rendered in world coordinates.
void
GpD3D7Scene::Draw(GVector viewpoint, GQuaternion angle, GFrustum viewFrustum, float geoLod)
{
	pGpD3D7Light light = mLightList;
	int i = 0;
	while(light != NULL)
	{
		light->Register(i);
		light = light->Next();
		i++;
	}
	while(i < 8)
	{
		mDevice->LightEnable(i,FALSE);
		i++;
	}
	
	pGpD3D7Sprite sprite = mSpriteList;
	while(sprite != NULL)
	{
		if(sprite->HasAlpha())
		{
			//Sprite has alpha -- it should be in the alpha list.
			pGpD3D7Sprite nextSprite = sprite->Next();
			
			//remove from current list.
			sprite->Remove();
			//add to alpha list
			sprite->AddToList(&mAlphaSpriteList);
			
			//process the next sprite
			sprite = nextSprite;
		}
		else
		{
			//Sprite has no alpha.  Draw it.
			sprite->Draw(viewpoint, angle, viewFrustum, geoLod);
			
			sprite = sprite->Next();
		}
	}
	
	sprite = mAlphaSpriteList;
	while(sprite != NULL)
	{
		sprite->Draw(viewpoint, angle, viewFrustum, geoLod);
		
		if(!sprite->HasAlpha())
		{
			//Sprite does not have alpha anymore -- move it back to the main list.
			pGpD3D7Sprite nextSprite = sprite->Next();
			
			//remove from current list.
			sprite->Remove();
			//add to main sprite list
			sprite->AddToList(&mSpriteList);
			
			//process next sprite
			sprite = nextSprite;
		}
		else
			sprite = sprite->Next();
	}
}

GAUGE3D_DECLARE_INTERFACES(GpD3D7Scene)
	GAUGE3D_DECLARE_BASECLASS(GScene)
GAUGE3D_END_INTERFACES

} //namespace Plugins
} //namespace GAUGE3D
