/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Direct3D 7 Rendering Plugin
 * Copyright (C) 2000 Steven Haworth
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: d3d7scene.h
 *      Package: Direct3D 7 Rendering Plugin
 *    Component: Scene
 *  Description: Basically just a list of sprites and lights
 * Dependencies: sprites, lights, etc.
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_DIRECT3D7_D3D7SCENE_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_DIRECT3D7_D3D7SCENE_H__INCLUDED__

#ifndef DIRECTINPUT_VERSION
#define DIRECTINPUT_VERSION 0x0700
#define DIRECT3D_VERSION    0x0700
#endif

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_SCENE_H__INCLUDED__
#include "gauge3d/renderer/scene.h"
#endif //redundant include guard

// local package -----------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_RENDERER_DIRECT3D7_D3D7LIGHT_H__INCLUDED__
#include "d3d7light.h"
#endif

#ifndef __GAUGE3D_PLUGINS_RENDERER_DIRECT3D7_D3D7SPRITE_H__INCLUDED__
#include "d3d7sprite.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpD3D7Scene: public GScene
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39782289)
private:
	pGpD3D7Light  mLightList;
	pGpD3D7Sprite mSpriteList;
	pGpD3D7Sprite mAlphaSpriteList;
	
public:
	// lifecycle -------------------------------------------------------
	GpD3D7Scene(IDirect3DDevice7 *device);
	virtual ~GpD3D7Scene();
	
	// operations ------------------------------------------------------
	//Add lights and sprites to the scene.
	virtual pGSprite NewSprite();
	virtual pGLight  NewLight();
	virtual pGCamera NewCamera();
	
	inline pGpD3D7Sprite SpriteList() { return mSpriteList; }
	inline pGpD3D7Light  LightList()  { return mLightList; }

	//Render the scene, assuming the projection and modelview matrices have
	// already been set up so that the scene can be rendered in world coordinates.
	void Draw(GVector viewpoint, GQuaternion angle, GFrustum viewFrustum, float geoLod);
	
private:
	IDirect3DDevice7 *mDevice;
};
typedef GSmartPointer<GpD3D7Scene> pGpD3D7Scene;
} //namespace Plugins
} //namespace GAUGE3D

#endif //included
