/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: d3d7texture.h
 *      Package: Direct3D Rendering Plugin
 *    Component: Texture
 *  Description: Represents a cached texture
 * Dependencies: GTexture
 *               GImage
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_DIRECT3D_D3D7TEXTURE_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_DIRECT3D_D3D7TEXTURE_H__INCLUDED__

#ifndef DIRECTINPUT_VERSION
#define DIRECTINPUT_VERSION 0x0700
#define DIRECT3D_VERSION    0x0700
#endif

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#define __GAUGE3D_STDDEFS_H__INCLUDED__
#endif //redundant include guard

// system ------------------------------------------------------------
#ifndef __SYSTEM_DDRAW_H__INCLUDED__
#include <ddraw.h>
#define __SYSTEM_DDRAW_H__INCLUDED__
#endif //redundant include guard

#ifndef __SYSTEM_D3D_H__INCLUDED__
#include <d3d.h>
#define __SYSTEM_D3D_H__INCLUDED__
#endif //redundant include guard

#ifndef __SYSTEM_VECTOR__INCLUDED__
#include <vector>
#define __SYSTEM_VECTOR__INCLUDED__
#endif //redundant include guard

// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_TEXTURE_H__INCLUDED__
#include "gauge3d/renderer/texture.h"
#define __GAUGE3D_RENDERER_TEXTURE_H__INCLUDED__
#endif

// local package -----------------------------------------------------
#ifndef __LOCAL_DXERROR_H__INCLUDED__
#include "dxerror.h"
#define __LOCAL_DXERROR_H__INCLUDED__
#endif

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpD3D7Texture: public GTexture
{
	GAUGE3D_SET_UID(0x00000048, 0x2168a11d, 0x39829b2d)
public:
	// lifecycle -------------------------------------------------------
	GpD3D7Texture(IDirectDraw7 *DD,IDirect3DDevice7 *Device);
	virtual ~GpD3D7Texture();
	
	// operations ------------------------------------------------------
	virtual void Image(pGImage image);

	virtual void Surface(pGSurface surface) {}
	
private:
	GError *mError;
	IDirectDraw7 *mDD;
	IDirect3DDevice7 *mDevice;

	IDirectDrawSurface7 *mSurface;

	int mTextureDepthLimit;

	std::vector<DDPIXELFORMAT> mTextureFormats;

	int CountBits(DWORD Mask);
	int CalcShift(DWORD Mask);
	LPDDPIXELFORMAT SelectTextureFormat(GImage::FormatDesc& SrcFormat);
	LPDDPIXELFORMAT FindTextureFormat(DWORD Flags,DWORD BitCount);
	void CopyImageToTexture(const GImage::FormatDesc& ImageFormat,const DDSURFACEDESC2& SurfaceFormat,const RECT& ImageRect,GArray<uint8> ImageData);

public:
	operator IDirectDrawSurface7 *(void) { return mSurface; }
};
typedef GSmartPointer<GpD3D7Texture> pGpD3D7Texture;

} //namespace Plugins
} //namespace GAUGE3D

#endif //included
