/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: glscene.h
 *      Package: OpenGL Rendering Plugin
 *    Component: Scene
 *  Description: Basically just a list of sprites and lights
 * Dependencies: sprites, lights, etc.
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLSCENE_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLSCENE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_SCENE_H__INCLUDED__
#include "gauge3d/renderer/scene.h"
#endif //redundant include guard

// local package -----------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLLIGHT_H__INCLUDED__
#include "gllight.h"
#endif

#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLSPRITE_H__INCLUDED__
#include "glsprite.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpGLScene: public GScene
{
	pGpGLLight  mLightList;
	pGpGLSprite mSpriteList;
	pGpGLSprite mAlphaSpriteList;
	
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x3978222a)
public:
	// lifecycle -------------------------------------------------------
	GpGLScene();
	virtual ~GpGLScene();
	
	// operations ------------------------------------------------------
	//Add lights and sprites to the scene.
	virtual pGSprite NewSprite();
	virtual pGLight  NewLight();
	virtual pGCamera NewCamera();
	
	inline pGpGLSprite SpriteList() {return mSpriteList;}
	inline pGpGLLight  LightList()  {return mLightList;}
	
	
	//Render the scene, assuming the projection and modelview matrices have
	// already been set up so that the scene can be rendered in world coordinates.
	void Draw(GVector viewpoint, GQuaternion angle, GFrustum viewFrustum, float geoLod);
};
typedef GSmartPointer<GpGLScene> pGpGLScene;

} //namespace Plugins
} //namespace GAUGE3D

#endif //included
