/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: gltexture.h
 *      Package: OpenGL Rendering Plugin
 *    Component: Texture
 *  Description: Represents a cached texture
 * Dependencies: GTexture
 *               GImage
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLTEXTURE_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLTEXTURE_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
#ifdef PLATFORM_WINDOWS
#include <windows.h>        //needed for gl.h  :(
#endif

#ifndef __SYSTEM_GL_GL_H__INCLUDED__
#include <GL/gl.h>
#define __SYSTEM_GL_GL_H__INCLUDED__
#endif //redundant include guard

// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_TEXTURE_H__INCLUDED__
#include "gauge3d/renderer/texture.h"
#endif

// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
class GpGLTexture: public GTexture
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x397860a4)
public:
	//Bind() sets up the pixel pipeline to apply the texture.
	void Bind();
	
	inline bool HasAlpha() {return mHasAlpha;}
	
	// lifecycle -------------------------------------------------------
	GpGLTexture();
	virtual ~GpGLTexture();
	
	// operations ------------------------------------------------------
	virtual void Surface(pGSurface surface);
	virtual void Image(pGImage image);
	
private:
	int mNumLayers;
	GLuint* mLayerIDs;  //GL texture numbers for the layers
	
	bool mHasAlpha;
	
	struct TexFormat
	{
		GLenum format;
		GLenum internalFormat;
		GLenum type;
		int bitsPerPixel;
	};
	
	TexFormat ImageFormat2TexFormat(GImage::FormatDesc imageFormat);
	
	void LoadTexture(pGImage image, int layer);
};
typedef GSmartPointer<GpGLTexture> pGpGLTexture;

} //namespace GAUGE3D

#endif //included
